#include <stdio.h>
#include <string.h>
#ifdef __MSDOS__
#include <dos.h>
#endif
#ifdef __unix__
#include <dirent.h>
#include <fnmatch.h>
#endif
#include <ctype.h>
#include "mlist.h"

static int do_help(char *, char *);
static int do_list(char *, char *);
static int do_users(char *, char *);
static int do_which(char *, char *);
static int do_subscribe(char *, char *);
static int do_unsubscribe(char *, char *);

struct {
	char *cmd;
	int (*func) (char *, char *);
} cmdlist[] = {
	{ "HELP", do_help },
	{ "LISTS", do_list },
	{ "USERS", do_users },
	{ "WHICH", do_which },
	{ "SUBSCRIBE", do_subscribe },
	{ "UNSUBSCRIBE", do_unsubscribe },
	{ "", NULL }
};

static int do_help(char *arg, char *arg2)
{
	char fname[30];

	build_path(fname, prgname, "HLP");
	if (copy_file(fname, mail_in))
		build_string(no_helpfile, mail_in, 1);

	return (0);
}

static int do_list(char *arg, char *arg2)
{
	print_info(mail_in);
	return (0);
}

static int do_users(char *arg, char *arg2)
{
	char fname[20];
	int i;

	if (arg == NULL || *arg == '\0') {
		build_string(no_arg, mail_in, 1);
		return (0);
	}

	if ((i = check_list(arg)) == -1) {
		build_string(list_noex, mail_in, 1);
		return (0);
	}

	lang.lst_owner = list_cfg[i]->owner;
	lang.owner_bbs = list_cfg[i]->owner_bbs;

	build_string(lusers, mail_in, 1);
	build_path(fname, arg, "USR");
	if (copy_file(fname, mail_in))
		return (-1);
	build_string(lowner, mail_in, 1);

	return (0);
}

#ifdef __unix__
static int do_which(char *arg, char *arg2)
{
	DIR	*dirp;
	struct dirent *dir;
	char dirname[80], name[80], *p;
	int header = 0;

	strcpy(dirname, prgname);
	strlwr(dirname);

	if ((dirp = opendir(dirname)) == NULL) {
		print_error("Can't opendir() %s", dirname);
		return (-1);
	}

	while ((dir = readdir(dirp)) != NULL) {
		if (!fnmatch("*.usr", dir->d_name, 0)) {
			strcpy(name, dir->d_name);
			p = strchr(name, '.');
			*p = '\0';
			if (check_user(name, lang.sender)) {
				if (!header) {
					build_string(you_are_subd, mail_in, 1);
					header = 1;
				}
				fprintf(mail_in, " %s\n", strupr(name));
			}
		}
	}

	if (!header) {
		build_string(you_are_subd_none, mail_in, 1);
	}

	closedir(dirp);
	return (0);
}
#else
static int do_which(char *arg, char *arg2)
{
	struct find_t ffblk;
	char name[80], *p;
	int done, header = 0;

	build_path(name, "*", "USR");
	done = _dos_findfirst(name, _A_NORMAL, &ffblk);
	while (!done) {
		strcpy(name, ffblk.name);
		p = strchr(name, '.');
		*p = '\0';
		if (check_user(name, lang.sender)) {
			if (!header) {
				build_string(you_are_subd, mail_in, 1);
				header = 1;
			}
			fprintf(mail_in, " %s\n", name);
		}
		done = _dos_findnext(&ffblk);
	}
	if (!header) {
		build_string(you_are_subd_none, mail_in, 1);
	}
	return (0);
}
#endif /* !__unix__ */

static int do_subscribe(char *arg, char *arg2)
{
	char fname[30], sender[7], home[7];
	int i, handled_by_owner = 0;

	if (arg == NULL || *arg == '\0') {
		build_string(no_arg, mail_in, 1);
		return (0);
	}

	if ((i = check_list(arg)) == -1) {
		build_string(list_noex, mail_in, 1);
		return (0);
	}

	lang.lst_owner = list_cfg[i]->owner;
	lang.owner_bbs = list_cfg[i]->owner_bbs;

	if (!strcmp(lang.sender, list_cfg[i]->owner)) {
		if (arg2[0]) {
			char *ptr, *ptr2;

			ptr = strtok(arg2, "@\t\n\r ");
			ptr2 = strtok(NULL, "@\t\n\r ");

			if (ptr == NULL || ptr2 == NULL) {
				build_string(address_missing, mail_in, 1);
				return (0);
			}

			strncpy(sender, ptr, 6);
			sender[6] = '\0';

			strncpy(home, ptr2, 6);
			home[6] = '\0';

			lang.sender = sender;
			lang.home = home;
		}
		handled_by_owner = 1;
	} else if (list_cfg[i]->access & ACC_PRIV) {
		build_string(lpriv, mail_in, 1);
		send_short_msg(list_cfg[i]->owner, list_cfg[i]->owner_bbs, bbs, owner_msg, tried_sub);
		return (0);
	}

	if (check_user(arg, lang.sender)) {
		build_string(alr_subd, mail_in, 1);
		return (0);
	}

	if (add_user(arg, lang.sender, lang.home))
		return (-1);

	build_string(subscribe_ok, mail_in, 1);

	if (!handled_by_owner) {
		build_path(fname, arg, "TXT");
		copy_file(fname, mail_in);
		send_short_msg(list_cfg[i]->owner, list_cfg[i]->owner_bbs, bbs, owner_msg, usr_subd);
	} else if (strcmp(lang.sender, list_cfg[i]->owner))
		send_short_msg(lang.sender, lang.home, bbs, subd, subd_by_owner);

	return (0);
}

static int do_unsubscribe(char *arg, char *arg2)
{
	int i, handled_by_owner = 0;
	char sender[7], home[7];

	if (arg == NULL || *arg == '\0') {
		build_string(no_arg, mail_in, 1);
		return (0);
	}

	if ((i = check_list(arg)) == -1) {
		build_string(list_noex, mail_in, 1);
		return (0);
	}

	lang.lst_owner = list_cfg[i]->owner;
	lang.owner_bbs = list_cfg[i]->owner_bbs;

	if (!strcmp(lang.sender, list_cfg[i]->owner)) {
		if (arg2[0]) {
			char *ptr, *ptr2;

			ptr = strtok(arg2, "@\t\n\r ");
			ptr2 = strtok(NULL, "@\t\n\r ");

			if (ptr == NULL || ptr2 == NULL) {
				build_string(address_missing, mail_in, 1);
				return (0);
			}

			strncpy(sender, ptr, 6);
			sender[6] = '\0';

			strncpy(home, ptr2, 6);
			home[6] = '\0';

			lang.sender = sender;
			lang.home = home;
		}
		handled_by_owner = 1;
	}

	if (!check_user(arg, lang.sender)) {
		build_string(not_subd, mail_in, 1);
		return (0);
	}

	if (remove_user(arg, lang.sender))
		return (-1);

	build_string(unsub_ok, mail_in, 1);

	if (!handled_by_owner)
		send_short_msg(list_cfg[i]->owner, list_cfg[i]->owner_bbs, bbs, owner_msg, usr_unsubd);
	else if (strcmp(lang.sender, list_cfg[i]->owner))
		send_short_msg(lang.sender, lang.home, bbs, unsubed, unsubd_by_owner);

	return (0);
}

int handle_commands(FILE * fp, char *sender, char *home_bbs)
{
	int i, ok = 0, errors = 0;
	char command[21], arg[81], arg2[81], line[128], *ptr;

	while (fgets(line, sizeof(line), fp) != NULL) {
		if ((ptr = strpbrk(line, "\r\n")) != NULL)
			*ptr = '\0';

		if (line[0] == '\0') {
			if (ok) {
				build_string(stopped_processing, mail_in, 1);
				break;
			} else
				continue;
		}

		if (!isalpha(line[0]) || !strncmp(line, "From", 4) || !strncmp(line, "To", 2))
			continue;

		if (!strncmp(line, "/EX", 3))
			break;

		arg[0] = arg2[0] = '\0';
		switch (sscanf(line, "%20s %80s %80[^\n]", command, arg, arg2)) {
		case 3:
			strupr(arg2);
		case 2:
			strupr(arg);
		case 1:
			strupr(command);
			break;
		default:
			continue;
		}

		lang.command = command;
		lang.list = arg;
		lang.lst_owner = unknown_esc;
		lang.owner_bbs = unknown_esc;
		lang.sender = sender;
		lang.home = home_bbs;
		lang.subject = unknown_esc;

		fprintf(mail_in, "\n--> %s\n", line);

		for (i = 0; cmdlist[i].func; i++) {
			if (strcmp(command, cmdlist[i].cmd))
				continue;
			if (cmdlist[i].func(arg, arg2))
				return (-1);
			ok++;
			errors = 0;
			break;
		}

		if (cmdlist[i].func == NULL) {
			build_string(unknown_cmd, mail_in, 1);
			if (errors++ == 5) {
				build_string(too_many_errors, mail_in, 1);
				break;
			}
		}
	}

	if (!ok)
		build_string(empty_body, mail_in, 1);

	return (0);
}
