#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include "mlist.h"

#ifdef NEED_STRUPR
char *strupr(char *str)
{
	char *p = str;

	do
		*str = toupper(*str);
	while (*(++str));

	return (p);
}

char *strlwr(char *str)
{
	char *p = str;

	do
		*str = tolower(*str);
	while (*(++str));

	return (p);
}
#endif

void print_error(char *fmt, ...)
{
	va_list ap;
	time_t now;
	struct tm *tm;

	time(&now);
	tm = localtime(&now);
	fprintf(stderr, "%2d-%02d-%02d %02d:%02d:%02d - ", tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
}

int copy_file(char *fname, FILE * fp_o)
{
	char buf[128];
	int len, last_was_cr = 0;
	FILE *fp;

	if ((fp = fopen(fname, "r")) == NULL)
		return (-1);

	while ((len = fread(buf, 1, sizeof(buf), fp)) > 0) {
		if (fwrite(buf, 1, len, fp_o) != len) {
			print_error("Error writing to %s (disk full?)\n", fname);
			return (-1);
		}
		last_was_cr = (buf[len - 1] == '\n');
	}

	if (!last_was_cr)
		fputc('\n', fp_o);

	return (0);
}

void build_path(char *buf, char *name, char *ext)
{
#ifdef __unix__
	sprintf(buf, "%s" PATH_SEP2 "%s.%s", prgname, name, ext);
	strlwr(buf);
#else
	sprintf(buf, "%s" PATH_SEP2 "%s.%s", prgname, name, ext);
#endif
}

void *do_malloc(size_t size)
{
	char *p;

	if ((p = malloc(size)) == NULL) {
		print_error("Out of memory (malloc)\n");
		exit(1);
	}
	return (p);
}

void *do_realloc(void *p, size_t size)
{
	if ((p = realloc(p, size)) == NULL) {
		print_error("Out of memory (realloc)\n");
		exit(1);
	}
	return (p);
}
